/*
 * Copyright (c) 2023–2024 DTB <trinity@trinity.moe>
 * Copyright (c) 2024 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <stdio.h> /* fprintf(3), stderr, NULL */
#include <stdlib.h> /* EXIT_FAILURE, EXIT_SUCCESS */
#include <string.h> /* memset(3), strchr(3) */
#include <sysexits.h> /* EX_USAGE */
#include <unistd.h> /* access(3), getopt(3), F_OK, R_OK, W_OK, X_OK */
#include <libfileis.h>

static char args[] = "bcdefgkprsuwxLS";
static char ops[(sizeof args) / (sizeof *args)];

char *program_name = "scrut";

int usage(char *s){
	fprintf(stderr, "Usage: %s [-%s] file...\n", s, args);
	return EX_USAGE;
}

int main(int argc, char *argv[]){
	int c;
	size_t i;
	char *p;

	if (argc > 0) { program_name = argv[0]; }
	if (argc < 2) { return usage(program_name); }

	memset(ops, '\0', sizeof ops);

	while ((c = getopt(argc, argv, args)) != -1)
		if ((p = strchr(args, c)) == NULL) { return usage(program_name); }
		else { ops[p - args] = c; }

	if (optind == argc) { return usage(program_name); }

	argv += optind;

	/* straighten out ops */
	for(i = 0, p = ops; i < (sizeof ops) / (sizeof *ops); ++i)
		if(ops[i] != '\0'){
			*p = ops[i];
			if(&ops[i] != p++)
				ops[i] = '\0';
		}

	do{
		if (!fileis_exists(*argv))
			return EXIT_FAILURE; /* doesn't exist or isn't stattable */

		for (i = 0; ops[i] != '\0'; ++i)
			if (ops[i] == 'e')
				continue;
			else if ((ops[i] == 'b' && !fileis_block(*argv))
					|| (ops[i] == 'c' && !fileis_char(*argv))
					|| (ops[i] == 'd' && !fileis_dir(*argv))
					|| (ops[i] == 'f' && !fileis_regular(*argv))
					|| (ops[i] == 'g' && !fileis_setgid(*argv))
					|| (ops[i] == 'k' && !fileis_vtx(*argv))
					|| (ops[i] == 'p' && !fileis_fifo(*argv))
					|| (ops[i] == 'r' && access(*argv, R_OK) != 0)
					|| (ops[i] == 'u' && !fileis_setuid(*argv))
					|| (ops[i] == 'w' && access(*argv, W_OK) != 0)
					|| (ops[i] == 'x' && access(*argv, X_OK) != 0)
					|| (ops[i] == 'L' && !fileis_link(*argv))
					|| (ops[i] == 'S' && !fileis_socket(*argv))
			) { return EXIT_FAILURE; }
	} while (*++argv != NULL);

	return EXIT_SUCCESS;
}
